#version 130
#ifdef VERTEX_SHADER_SOURCE

uniform mat4 mvMat;
uniform mat4 projMat;
uniform mat4 modelMat;
uniform mat3 inverseModelMat;

in vec3 inVertex;
in vec2 inTexCoord;
in vec3 inColor;
in vec3 inNormal;
out vec4 color;
out vec2 texCoord;
out vec4 pointInWorld;
out vec3 normal;
out vec4 worldVertex;

void main(void)
{
	mat4 finalMatrix = projMat * mvMat;
	gl_Position = finalMatrix * vec4(inVertex, 1.0);
	pointInWorld = mvMat * vec4(inVertex, 1.0);
	color = vec4(inColor, 1.0);
	texCoord = inTexCoord;

	mat3 normalMatrix = transpose(inverseModelMat);
	normal = inNormal;//normalize(normalMatrix * inNormal);
	worldVertex = modelMat * vec4(inVertex, 1.0);
}
#endif

#ifdef FRAGMENT_SHADER_SOURCE
uniform vec3 lightDirection;
uniform sampler2D texture0;
//uniform sampler2D texture1;
//uniform sampler2D shadowMap;

in vec4 color;
in vec2 texCoord;
in vec4 pointInWorld;
in vec3 normal;
in vec4 worldVertex;
//in vec2 refractionTexCoord;
//in vec4 shadowCoord;
//out vec4 outColor;

void main(void)
{
	if(worldVertex.z <= 0.0)
		discard;

	vec4 fragColor = color*texture(texture0, texCoord.st);
	vec4 textureColor = fragColor;
	if(fragColor.a < 0.3)
		discard;

	//DIRECTIONAL LIGHT
	vec3 ambient = 0.005 * fragColor.rgb;
	float diffuseCoefficientDirectional = max(0.0, dot(normal, -lightDirection.xyz));
	vec3 diffuse = diffuseCoefficientDirectional * fragColor.rgb;

	fragColor.rgb = diffuse;//mix(ambient+diffuse, ambient, clamp(-worldVertex.z/20.0, 0.0, 1.0));//The deeper we are, the less sunlight touches the surface
	//END DIRECTIONAL LIGHT

	//FOG
	float blendFactor = clamp(length(pointInWorld)/80.0, 0.0, 1.0);
	//blendFactor = 1.0 - (1.0 / (pow(2.71828, blendFactor * 1.0)));
	blendFactor = pow(blendFactor, 1.4);

	vec4 fogColor = vec4(0.0,0.35,0.5,1);//vec4(0.2078, 0.4784, 0.5098, 1.0);
	vec4 outColor = fragColor;
	//END FOG

	vec3 gamma = vec3(1.0/2.2);
    outColor = vec4(pow(outColor.rgb, gamma), 1.0);
	gl_FragColor = outColor;
}
#endif
