
struct PSInput
{
	float4 position : SV_POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
	float3 normal : NORMAL;
	float4 viewPosition : POSITION;
	float4 worldVertex : WORLDPOSITION;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 mvMat;
	float4x4 projMat;
	float4x4 modelMat;
	float3x3 inverseModelMat;
	float3 lightDirection;
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0));
	result.viewPosition = mul(mvMat, float4(position, 1.0));
	result.uv = uv;
	result.color = float4(color, 1.0);

	float3x3 normalMatrix = transpose(inverseModelMat);
	result.normal = normal;//normalize(mul(normalMatrix, normal));
	result.worldVertex = mul(modelMat, float4(position, 1.0));

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	if(input.worldVertex.z <= 0.0)
		discard;
		
	float4 fragColor = input.color * g_texture.Sample(g_sampler, input.uv);
	float4 textureColor = fragColor;
	if(fragColor.a < 0.3)
		discard;

	//DIRECTIONAL LIGHT
	float3 ambient = 0.005 * fragColor.rgb;
	float diffuseCoefficientDirectional = max(0.0, dot(input.normal, -lightDirection.xyz));
	float3 diffuse = diffuseCoefficientDirectional * fragColor.rgb;

	fragColor.rgb = diffuse;//lerp(ambient+diffuse, ambient, clamp(-input.worldVertex.z/20.0, 0.0, 1.0));//The deeper we are, the less sunlight touches the surface
	//END DIRECTIONAL LIGHT

	//FOG
	float blendFactor = clamp(length(input.viewPosition)/80.0, 0.0, 1.0);
	//blendFactor = 1.0 - (1.0 / (pow(2.71828, blendFactor * 1.0)));
	blendFactor = pow(blendFactor, 1.4);

	float4 fogColor = float4(0.0,0.35,0.5,1);//float4(0.2078, 0.4784, 0.5098, 1.0);

	float4 outColor = fragColor;
	//END FOG

	float3 gamma = float3(1.0/2.2, 1.0/2.2, 1.0/2.2);
	outColor = float4(pow(abs(outColor.rgb), gamma), 1.0);

	return outColor;
}
