//*********************************************************
//
// Copyright (c) Microsoft. All rights reserved.
// This code is licensed under the MIT License (MIT).
// THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
// IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
// PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

struct PSInput
{
	float4 position : SV_POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 mvMat;
	float4x4 projMat;
};

PSInput VSMain(float2 position : POSITION, float2 uv : TEXCOORD, float4 color : COLOR)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0f, 1.0f));
	result.uv = uv;
	result.color = color;

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	float4 outColor = input.color * g_texture.Sample(g_sampler, input.uv);
	if(outColor.a < 0.3)
		discard;
	return outColor;
}
