
struct PSInput
{
	float4 position : SV_POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
	float3 normal : NORMAL;
	float4 viewPosition : POSITION;
	float4 worldVertex : WORLDPOSITION;
	//float2 refractionTexCoord : REFRACTIONTEXCOORD;
	//float4 shadowCoord : SHADOWPOSITION;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);
//Texture2D g_textureRefraction : register(t1);
//SamplerState g_samplerRefraction : register(s1);
//Texture2D g_shadowMap : register(t2);
//SamplerState g_samplerShadowMap : register(s2);

cbuffer cb0 : register(b0)
{
	float4x4 mvMat;
	float4x4 projMat;
	float4x4 modelMat;
	float3x3 inverseModelMat;
	//float4x4 refractionModelMat;
	//float4x4 shadowMat;
	//float3 spotPosition;
	//float3 spotDirection;
	float3 lightDirection;
	float cameraAltitude;
	float waterAltitude;
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0));
	result.viewPosition = mul(mvMat, float4(position, 1.0));
	result.uv = uv;
	result.color = float4(color, 1.0);

	/*
	float4 refractionPos = mul(refractionModelMat, float4(position, 1.0));
	result.refractionTexCoord = (refractionPos.xy * 0.25);//+offset;
	*/
	float3x3 normalMatrix = transpose(inverseModelMat);
	result.normal = normal;//normalize(mul(normalMatrix, normal));
	result.worldVertex = mul(modelMat, float4(position, 1.0));

	//float4x4 shadowModelMat = mul(shadowMat, modelMat);
	//result.shadowCoord = mul(shadowModelMat, float4(position, 1.0));

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	if((cameraAltitude < waterAltitude && input.worldVertex.z < waterAltitude)
		|| (cameraAltitude > waterAltitude && input.worldVertex.z > waterAltitude))
		discard;


	float4 fragColor = input.color * g_texture.Sample(g_sampler, input.uv);
	float4 textureColor = fragColor;
	if(fragColor.a < 0.3)
		discard;

	//DIRECTIONAL LIGHT
	float3 ambient = 0.005 * fragColor.rgb;
	float diffuseCoefficientDirectional = max(0.0, dot(input.normal, -lightDirection.xyz));
	float3 diffuse = diffuseCoefficientDirectional * fragColor.rgb;

	fragColor.rgb = lerp(ambient+diffuse, ambient, clamp(-input.worldVertex.z/20.0, 0.0, 1.0));//The deeper we are, the less sunlight touches the surface
	//END DIRECTIONAL LIGHT
	/*
	//APPLY CAUSTICS
	if(input.worldVertex.z < waterAltitude && diffuseCoefficientDirectional > 0.0)
	{
		float causticColor = g_textureRefraction.Sample(g_samplerRefraction, input.refractionTexCoord).r;
		causticColor *= 3;
		float4 newFragColor = fragColor * (causticColor);
		fragColor = lerp(newFragColor, fragColor, clamp(0.0, 0.0, 1.0));
	}
	//END APPLY CAUSTICS
	*/
	/*
	//POINT LIGHT
	float3 surfaceToLight = normalize(spotPosition - input.worldVertex.xyz);
	float distanceToLight = length(spotPosition - input.worldVertex.xyz);
	float diffuseCoefficient = max(0.0, dot(input.normal, surfaceToLight));
	float lightAttenuation = 0.01;
	float3 lightIntensity = float3(1.0, 1.0, 1.0);
	float attenuation = 1.0 / (1.0 + lightAttenuation * pow(distanceToLight, 2));

	float spotExteriorAngle = 40.0;
	float spotInteriorAngle = 30.0;
	float lightToSurfaceAngle = acos(dot(-surfaceToLight, normalize(spotDirection))) / 0.017453292519943;
	if(lightToSurfaceAngle > spotInteriorAngle)
	{
		float attenuationFactor = clamp(lerp(0.0, 1.0, (lightToSurfaceAngle-spotInteriorAngle)/(spotExteriorAngle-spotInteriorAngle)), 0.0, 1.0);
		attenuationFactor = 1.0 - attenuationFactor;
		attenuation *= attenuationFactor;
	}
	if(lightToSurfaceAngle <= spotExteriorAngle)
	{
		float bias = 0.005;
		float3 spotShadowCoord = input.shadowCoord.xyz/input.shadowCoord.w;
		spotShadowCoord.xy = 0.5 * spotShadowCoord.xy + 0.5;
		spotShadowCoord.y = 1.0 - spotShadowCoord.y;//La texture est inversee sur l'axe y: on la remet a l'endroit
		float distanceToNearestOccluder = g_shadowMap.Sample(g_samplerShadowMap, spotShadowCoord.xy).r;
		float distanceLightToFragment = (spotShadowCoord.z - bias/input.shadowCoord.w);
		if(distanceToNearestOccluder < distanceLightToFragment)
			attenuation = 0.0;
	}

	fragColor.rgb += lightIntensity*diffuseCoefficient*attenuation * textureColor.rgb;
	//END POINT LIGHT
	*/

	//FOG
	float blendFactor = clamp(length(input.viewPosition)/80.0, 0.0, 1.0);
	//blendFactor = 1.0 - (1.0 / (pow(2.71828, blendFactor * 1.0)));
	blendFactor = pow(blendFactor, 1.4);

	float4 fogColor = float4(0.0,0.35,0.5,1);//float4(0.2078, 0.4784, 0.5098, 1.0);

	float4 outColor;
	if(input.worldVertex.z < waterAltitude)
		outColor = lerp(fragColor, fogColor, blendFactor);
	else
		outColor = fragColor;
	//END FOG

	float3 gamma = float3(1.0/2.2, 1.0/2.2, 1.0/2.2);
	outColor = float4(pow(abs(outColor.rgb), gamma), 1.0);

	return outColor;
}
