#version 130
#ifdef VERTEX_SHADER_SOURCE

uniform mat4 mvMat;
uniform mat4 modelMat;
uniform mat4 projMat;

in vec3 inVertex;
in vec2 inTexCoord;
in vec3 inColor;
in vec3 inNormal;
out vec4 color;
out vec4 position;
out vec2 texCoord;
out vec4 normal;
out vec4 viewPosition;
out vec4 worldPosition;

void main(void)
{
	mat4 finalMatrix = projMat * mvMat;
	position = finalMatrix * vec4(inVertex, 1.0);
	gl_Position = position;
	vec4 localWorldPosition = modelMat * vec4(inVertex, 1.0);
	worldPosition = localWorldPosition;//Certaines implementations n'acceptent pas qu'on utilise un "out" dans les calculs
	texCoord = localWorldPosition.xy / 30.0;
	viewPosition = mvMat * vec4(inVertex, 1.0);
	color = vec4(inColor, 1.0);
	normal = vec4(inNormal, 1.0);
}
#endif

#ifdef FRAGMENT_SHADER_SOURCE
uniform sampler2D texture0;//reflection//corrupted(WE2020)
uniform sampler2D texture1;//refraction//reflection(WE2020)
uniform sampler2D texture2;//dudv
uniform sampler2D texture3;//normal map
uniform sampler2D texture4;//refraction depth//reflection depth corrupted(WE2020)
uniform sampler2D texture5;//reflection depth

uniform vec3 cameraPosition;
uniform vec3 lightDirection;
uniform vec3 playerPosition;
uniform float dudvOffset;
uniform vec2 limits;

in vec4 color;
in vec4 position;
in  vec2 texCoord;
in vec4 normal;
in vec4 viewPosition;
in vec4 worldPosition;
out vec4 outColor;

void main(void)
{
	vec2 reflectionCoord = position.xy/position.w;
	reflectionCoord = 0.5 * reflectionCoord + 0.5;
	float waveStrength = 0.03f;

	float near = 0.2;
	float far = 1000.0;
	//(vec2(1.0-reflectionCoord.x, reflectionCoord.y)) instead of reflectionCoord
	//because for this game, reflection and refraction textures are actually the same
	float depthRefraction = 0;
	float limitWidth = 1.5;
	if(worldPosition.x < limits.x - limitWidth)
		depthRefraction = texture(texture4, (vec2(1.0-reflectionCoord.x, reflectionCoord.y))).z;
	else if(worldPosition.x > limits.x)
		depthRefraction = texture(texture5, (vec2(1.0-reflectionCoord.x, reflectionCoord.y))).z;
	else
	{
		float a = texture(texture4, (vec2(1.0-reflectionCoord.x, reflectionCoord.y))).z;
		float b = texture(texture5, (vec2(1.0-reflectionCoord.x, reflectionCoord.y))).z;
		depthRefraction = mix(a, b, (worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}
	float floorDistance = 2.0 * near * far /(far + near - (2.0 * depthRefraction - 1.0) * (far - near));
	depthRefraction = gl_FragCoord.z;
	float waterDistance = 2.0 * near * far /(far + near - (2.0 * depthRefraction - 1.0) * (far - near));
	depthRefraction = floorDistance - waterDistance;

	reflectionCoord.x = 1.0 - reflectionCoord.x;
	float depthReflection = 0;
	if(worldPosition.x < limits.x - limitWidth)
		depthReflection = texture(texture4, reflectionCoord).z;
	else if(worldPosition.x > limits.x)
		depthReflection = texture(texture5, reflectionCoord).z;
	else
	{
		float a = texture(texture4, reflectionCoord).z;
		float b = texture(texture5, reflectionCoord).z;
		depthReflection = mix(a, b, (worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}
	reflectionCoord.x = 1.0 - reflectionCoord.x;
	floorDistance = 2.0 * near * far /(far + near - (2.0 * depthReflection - 1.0) * (far - near));
	depthReflection = gl_FragCoord.z;
	waterDistance = 2.0 * near * far /(far + near - (2.0 * depthReflection - 1.0) * (far - near));
	depthReflection = floorDistance - waterDistance;

	//Coordonnees de la map de distortion
	vec2 dudvCoord1 = texture(texture2, vec2(texCoord.x + dudvOffset, texCoord.y)).rg * 0.1;
	dudvCoord1 = texCoord + vec2(dudvCoord1.x, dudvCoord1.y + dudvOffset);
	vec2 dudvCoord = (texture(texture2, dudvCoord1).rg * 2.0 - 1.0) * waveStrength;

	//Diminue la distortion autour du joueur
	//Pour le cas ou il est a moitie immerge
	float playerRadius = 0.7;
	if(length(playerPosition - worldPosition.xyz) < playerRadius)
		dudvCoord = vec2(0.0, 0.0);
	else
		dudvCoord *= mix(0.0, 1.0, clamp((length(playerPosition - worldPosition.xyz)-playerRadius), 0.0, 1.0));

	vec2 dudvCoordRefraction = dudvCoord * clamp(depthRefraction/5, 0.0, 1.0);
	vec2 dudvCoordReflection = dudvCoord * clamp(depthReflection/5, 0.0, 1.0);

	//(vec2(1.0-reflectionCoord.x, reflectionCoord.y)) instead of reflectionCoord.xy
	//because for this game, reflection and refraction textures are actually the same
	vec4 fragColorRefraction;
	if(worldPosition.x < limits.x - limitWidth)
		fragColorRefraction = texture(texture0, clamp((vec2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
	else if(worldPosition.x > limits.x)
		fragColorRefraction = texture(texture1, clamp((vec2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
	else
	{
		vec4 a = texture(texture0, clamp((vec2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
		vec4 b = texture(texture1, clamp((vec2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
		fragColorRefraction = mix(a, b, (worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}
	reflectionCoord.x = 1.0 - reflectionCoord.x;
	vec4 fragColor;
	if(worldPosition.x < limits.x - limitWidth)
		fragColor = texture(texture0, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
	else if(worldPosition.x > limits.x)
		fragColor = texture(texture1, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
	else
	{
		vec4 a = texture(texture0, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
		vec4 b = texture(texture1, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
		fragColor = mix(a, b, (worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}

	vec3 vertexToCamera = cameraPosition - worldPosition.xyz;
	vertexToCamera = normalize(vertexToCamera);
	vec3 normalColor = texture(texture3, dudvCoord1).rgb;
	normalColor.xy = normalColor.xy * 2.0 - 1.0;
	normalColor = normalize(normalColor);
	vec3 waterNormal = normal.xyz * normalColor;
	//TODO: fogColor doit etre un uniform
	vec4 fogColor = vec4(0.0,0.35,0.5,1);
	fragColor = (fragColor * 0.9) + (fogColor * 0.1);//teint l'eau avec la couleur du fog, pour que l'eau ne puisse pas ressembler a un miroir
	fragColor = mix(fragColor, fragColorRefraction, mix(0.1, 0.9, clamp(dot(vertexToCamera, normalize(vec3(waterNormal.xy, waterNormal.z * 3.0))), 0.0, 1.0)));

	vec3 sunLightColor = vec3(0.5, 0.5, 0.5);
	float shineDamper = 20.0;
	float reflectivity = 0.6;
	vec3 reflectedLight = reflect(normalize(lightDirection), normalColor);
	float specular = max(dot(reflectedLight, vertexToCamera), 0.0);
	specular = pow(specular, shineDamper);
	vec3 specularHighlights = sunLightColor * specular * reflectivity;

	//FOG
	float blendFactor = clamp(length(viewPosition)/80.0, 0.0, 1.0);
	blendFactor = pow(blendFactor, 1.4);
	vec3 gamma = vec3(1.0/2.2, 1.0/2.2, 1.0/2.2);
	fogColor = vec4(pow(abs(fogColor.rgb), gamma), 1.0);
	if(normal.z < 0.0)
		outColor = mix(fragColor, fogColor, blendFactor);
	else
		outColor = fragColor;
	//outColor += vec4(specularHighlights, 0.0);
	outColor += vec4(specularHighlights.r < sunLightColor.r * 0.5 ? specularHighlights/3 : pow(specularHighlights, vec3(0.6)), 0.0);
}
#endif
