
struct PSInput
{
	float4 position : SV_POSITION;
	float4 positionForReflection : POSITIONT;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
	float4 normal : NORMAL;
	float4 viewPosition : POSITION;
	float4 worldPosition : POSITION1;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);
Texture2D g_textureReflection : register(t1);
SamplerState g_samplerReflection : register(s1);
Texture2D g_textureDudv : register(t2);
SamplerState g_samplerDudv : register(s2);
Texture2D g_textureNormalMap : register(t3);
SamplerState g_samplerNormalMap : register(s3);
Texture2D g_textureReflectionCorruptedDepth : register(t4);
SamplerState g_samplerReflectionCorruptedDepth : register(s4);
Texture2D g_textureReflectionDepth : register(t5);
SamplerState g_samplerReflectionDepth : register(s5);

cbuffer cb0 : register(b0)
{
	float4x4 mvMat;
	float4x4 modelMat;
	float4x4 projMat;
	float3 cameraPosition;
	float3 lightDirection;
	float3 playerPosition;
	float dudvOffset;
	float2 limits;
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0f));
	result.positionForReflection = result.position;//SV_POSITION is automatically modified for rasterization, so we need another position for the refkection
	result.worldPosition = mul(modelMat, float4(position, 1.0f));
	result.uv = result.worldPosition.xy / 30.0;
	result.viewPosition = mul(mvMat, float4(position, 1.0));
	result.color = float4(color[0], color[1], color[2], 1.0);
	result.normal = float4(normal[0], normal[1], normal[2], 1.0);

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	float2 reflectionCoord = input.positionForReflection.xy/input.positionForReflection.w;
	reflectionCoord = 0.5 * reflectionCoord.xy + 0.5;
	//reflectionCoord.y = 1.0 - reflectionCoord.y;//y: La texture est inversee sur l'axe y: on la remet a l'endroit
	float waveStrength = 0.03f;

	float near = 0.2;
	float far = 1000.0;
	//(float2(1.0-reflectionCoord.x, reflectionCoord.y)) instead of reflectionCoord
	//because for this game, reflection and refraction textures are actually the same
	float limitWidth = 1.5;
	float depthRefraction = 0;
	if(input.worldPosition.x < limits.x - limitWidth)
		depthRefraction = g_textureReflectionCorruptedDepth.Sample(g_samplerReflectionCorruptedDepth, (float2(1.0-reflectionCoord.x, reflectionCoord.y))).r;
	else if(input.worldPosition.x > limits.x)
		depthRefraction = g_textureReflectionDepth.Sample(g_samplerReflectionDepth, (float2(1.0-reflectionCoord.x, reflectionCoord.y))).r;
	else
	{
		float a = g_textureReflectionCorruptedDepth.Sample(g_samplerReflectionCorruptedDepth, (float2(1.0-reflectionCoord.x, reflectionCoord.y))).r;
		float b = g_textureReflectionDepth.Sample(g_samplerReflectionDepth, (float2(1.0-reflectionCoord.x, reflectionCoord.y))).r;
		depthRefraction = lerp(a, b, (input.worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}
	float floorDistance = 2.0 * near * far /(far + near - (2.0 * depthRefraction - 1.0) * (far - near));
	depthRefraction = input.position.z;
	float waterDistance = 2.0 * near * far /(far + near - (2.0 * depthRefraction - 1.0) * (far - near));
	depthRefraction = floorDistance - waterDistance;

	reflectionCoord.x = 1.0 - reflectionCoord.x;
	float depthReflection = 0;
	if(input.worldPosition.x < limits.x - limitWidth)
		depthReflection = g_textureReflectionCorruptedDepth.Sample(g_samplerReflectionCorruptedDepth, reflectionCoord).r;
	else if(input.worldPosition.x > limits.x)
		depthReflection = g_textureReflectionDepth.Sample(g_samplerReflectionDepth, reflectionCoord).r;
	else
	{
		float a = g_textureReflectionCorruptedDepth.Sample(g_samplerReflectionCorruptedDepth, reflectionCoord).r;
		float b = g_textureReflectionDepth.Sample(g_samplerReflectionDepth, reflectionCoord).r;
		depthReflection = lerp(a, b, (input.worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}
	reflectionCoord.x = 1.0 - reflectionCoord.x;
	floorDistance = 2.0 * near * far /(far + near - (2.0 * depthReflection - 1.0) * (far - near));
	depthReflection = input.position.z;
	waterDistance = 2.0 * near * far /(far + near - (2.0 * depthReflection - 1.0) * (far - near));
	depthReflection = floorDistance - waterDistance;

	//Coordonnees de la map de distortion
	float2 dudvCoord1 = g_textureDudv.Sample(g_samplerDudv, float2(input.uv.x + dudvOffset, input.uv.y)).rg * 0.1;
	dudvCoord1 = input.uv + float2(dudvCoord1.x, dudvCoord1.y + dudvOffset);
	float2 dudvCoord = (g_textureDudv.Sample(g_samplerDudv, dudvCoord1).rg * 2.0 - 1.0) * waveStrength;

	//Diminue la distortion autour du joueur
	//Pour le cas ou il est a moitie immerge
	float playerRadius = 0.7;
	if(length(playerPosition - input.worldPosition.xyz) < playerRadius)
		dudvCoord = float2(0.0, 0.0);
	else
		dudvCoord *= lerp(0.0, 1.0, clamp((length(playerPosition - input.worldPosition.xyz)-playerRadius), 0.0, 1.0));

	float2 dudvCoordRefraction = dudvCoord * clamp(depthRefraction/5, 0.0, 1.0);
	float2 dudvCoordReflection = dudvCoord * clamp(depthReflection/5, 0.0, 1.0);

	//(float2(1.0-reflectionCoord.x, reflectionCoord.y)) instead of reflectionCoord.xy
	//because for this game, reflection and refraction textures are actually the same
	float4 fragColorRefraction = 0;
	if(input.worldPosition.x < limits.x - limitWidth)
		fragColorRefraction = g_texture.Sample(g_sampler, clamp((float2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
	else if(input.worldPosition.x > limits.x)
		fragColorRefraction = g_textureReflection.Sample(g_samplerReflection, clamp((float2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
	else
	{
		float4 a = g_texture.Sample(g_sampler, clamp((float2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
		float4 b = g_textureReflection.Sample(g_samplerReflection, clamp((float2(1.0-reflectionCoord.x, reflectionCoord.y))+dudvCoordRefraction, 0.001, 0.999));
		fragColorRefraction = lerp(a, b, (input.worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}
	reflectionCoord.x = 1.0 - reflectionCoord.x;
	float4 fragColor;
	if(input.worldPosition.x < limits.x - limitWidth)
		fragColor = g_texture.Sample(g_sampler, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
	else if(input.worldPosition.x > limits.x)
		fragColor = g_textureReflection.Sample(g_samplerReflection, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
	else
	{
		float4 a = g_texture.Sample(g_sampler, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
		float4 b = g_textureReflection.Sample(g_samplerReflection, clamp(reflectionCoord.xy+dudvCoordReflection, 0.001, 0.999));
		fragColor = lerp(a, b, (input.worldPosition.x - (limits.x - limitWidth))/limitWidth);
	}

	float3 vertexToCamera = cameraPosition - input.worldPosition.xyz;
	vertexToCamera = normalize(vertexToCamera);
	float3 normalColor = g_textureNormalMap.Sample(g_samplerNormalMap, dudvCoord1).rgb;
	normalColor.xy = normalColor.xy * 2.0 - 1.0;
	normalColor = normalize(normalColor);
	float3 waterNormal = input.normal.xyz * normalColor;
	//TODO: fogColor doit etre un uniform
	float4 fogColor = float4(0.0,0.35,0.5,1);
	fragColor = (fragColor * 0.9) + (fogColor * 0.1);//teint l'eau avec la couleur du fog, pour que l'eau ne puisse pas ressembler a un miroir
	fragColor = lerp(fragColor, fragColorRefraction, lerp(0.1, 0.9, clamp(dot(vertexToCamera, normalize(float3(waterNormal.xy, waterNormal.z * 3.0))), 0.0, 1.0)));

	float3 sunLightColor = float3(0.5, 0.5, 0.5);
	float shineDamper = 20.0;
	float reflectivity = 0.6;
	float3 reflectedLight = reflect(normalize(lightDirection), normalColor);
	float specular = max(dot(reflectedLight, vertexToCamera), 0.0);
	specular = pow(specular, shineDamper);
	float3 specularHighlights = sunLightColor * specular * reflectivity;

	//FOG
	float blendFactor = clamp(length(input.viewPosition)/80.0, 0.0, 1.0);
	blendFactor = pow(blendFactor, 1.4);
	float3 gamma = float3(1.0/2.2, 1.0/2.2, 1.0/2.2);
	fogColor = float4(pow(abs(fogColor.rgb), gamma), 1.0);
	float4 outColor;
	if(input.normal.z < 0.0)
		outColor = lerp(fragColor, fogColor, blendFactor);
	else
		outColor = fragColor;
	//outColor += float4(specularHighlights, 0.0);
	outColor += float4(specularHighlights.r < sunLightColor.r * 0.5 ? specularHighlights/3 : pow(specularHighlights, 0.6), 0.0);
	//outColor.a = clamp(min(depthReflection, depthRefraction)/5, 0.0, 1.0);
	return outColor;
}
